﻿
Imports System
Imports MediaCy.IQL.Engine
Imports MediaCy.IQL.Application
Imports MediaCy.IQL.Calibrations
Imports MediaCy.Commands.LineProfile
Imports MediaCy.IQL.Display.Overlays
Imports MediaCy.Addins.Scripting.Workflow

Public Enum OverlayTools
    None = mcOverlayTool.mcotNone
    Edit = mcOverlayTool.mcotEdit
    Polyline = mcOverlayTool.mcotPolyline
    Reference = mcOverlayTool.mcotManualReference
    Peaks = mcOverlayTool.mcotManualPeaks
End Enum

Public Class AppForm

    Private WithEvents _app As McApplication = McApplication.ThisApplication
    Private WithEvents _panels As McPanels = McApplication.ThisApplication.Panels
    Private WithEvents _lineProfile As McLineProfileEngine = Nothing

    Private _bInit As Boolean
    Private _iStep As Integer
    Private _OverlayTool As OverlayTools
    Private _iAnnuliMarksCount As Integer
    Private _iCirculiMarksCount As Integer
    Private _bZoomIn As Boolean
    Private _iZoomStep As Integer
    Private _ZoomPoints As New Collections.Generic.List(Of Drawing.PointF)

    Private _bSpatial As Boolean = False
    Private _sSpatial As String = ""
    Private _bIntensity As Boolean = False
    Private _sIntensity As String = ""
    Private _sTransectName As String = ""
    Private _cTransectColor As Drawing.Color = drawing.Color.Red
    Private _dMagnification As Double = 5.0
    Private _sAnnuliPrefix As String = "Year"
    Private _cAnnuliColor As Drawing.Color = Drawing.Color.LimeGreen
    Private _sCirculiName As String = "Growth"
    Private _cCirculiColor As Drawing.Color = Drawing.Color.Yellow

    'Public Sub New()
    ' Initialization code goes here
    'End Sub

    'Protected Overrides Sub Finalize()
    ' Cleanup code goes here
    'End Sub

#Region "Properties"

    ' Provides access to underlying Control
    Public ReadOnly Property Control As System.Windows.Forms.UserControl
        Get
            Return MyControl
        End Get
    End Property

    Public ReadOnly Property LineProfile As McLineProfileEngine
        Get
            Return _lineProfile
        End Get
    End Property

    Public ReadOnly Property SpatialName As String
        Get
            If _bSpatial AndAlso Not String.IsNullOrEmpty(_sSpatial) Then
                Return _sSpatial
            Else
                Return ""
            End If
        End Get
    End Property

    Public ReadOnly Property IntensityName As String
        Get
            If _bIntensity AndAlso Not String.IsNullOrEmpty(_sIntensity) Then
                Return _sIntensity
            Else
                Return ""
            End If
        End Get
    End Property

    Public ReadOnly Property TransectName As String
        Get
            Return _sTransectName
        End Get
    End Property

    Public ReadOnly Property TransectColor As Drawing.Color
        Get
            Return _cTransectColor
        End Get
    End Property

    Public ReadOnly Property Magnification As Double
        Get
            Return _dMagnification
        End Get
    End Property

    Public ReadOnly Property Pan As Double
        Get
            Return _ZoomPoints(_iZoomStep).X
        End Get
    End Property

    Public ReadOnly Property Scroll As Double
        Get
            Return _ZoomPoints(_iZoomStep).Y
        End Get
    End Property

    Public ReadOnly Property Data1 As String
        Get
            Dim s As String = ""

            If _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1 Then
                s &= String.Format("Image Name: {0}", _lineProfile.Document.DisplayName) & vbNewLine
            End If

            Return s.Trim
        End Get
    End Property

    Public ReadOnly Property Data2 As String()
        Get
            Dim ls As New System.Collections.Generic.List(Of String), s As String, sc As String = "px", ic As String = "lum"
            Dim pd As McProfileData, ed As McEdgesData, ec As McEdgeClass, dA() As Double, dC() As Double, dStep As Double
            Dim k As Integer, d As Double, l As Double, fs As String = GetFormatString
            Dim index As Integer, dp As MediaCy.IQL.ObjectManager.DOUBLEPOINT, intensity As Double
            Dim ar As Array

            If _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1 Then
                If _lineProfile.SpatialCalibration AndAlso _lineProfile.Image.SpatialCalibration IsNot Nothing Then sc = _lineProfile.Image.SpatialCalibration.UnitAbbrev
                If _lineProfile.IntensityCalibration AndAlso _lineProfile.Image.IntensityCalibration IsNot Nothing Then ic = _lineProfile.Image.IntensityCalibration.UnitAbbrev

                pd = _lineProfile.ProfileData(0)
                dStep = _lineProfile.LengthPerSampleDesired
                ed = pd.EdgesData(mcEdgeTypes.mcetReference)
                dA = ed.Distances(0)

                If dA.Length > 0 Then
                    s = "Periodic #:" & vbTab &
                        String.Format("Position ({0}):", sc) & vbTab &
                        String.Format("Location X ({0}):", "px") & vbTab &
                        String.Format("Location Y ({0}):", "px") & vbTab &
                        String.Format("Intensity ({0}):", ic) & vbTab &
                        String.Format("Distance ({0}):", sc) & vbTab &
                        "Label:" & vbNewLine

                    k = 1 : l = 0
                    For Each d In dA
                        ec = ed.EdgeClass(0, d)
                        index = CInt(d / dStep)
                        dp = pd.Points(index)
                        ar = pd.Values
                        intensity = ar(index)

                        s &= k.ToString & vbTab &
                            Format(d, fs) & vbTab &
                            Format(dp.X, fs) & vbTab &
                            Format(dp.Y, fs) & vbTab &
                            Format(intensity, fs) & vbTab &
                            Format(d - l, fs) & vbTab &
                            ec.Text & vbNewLine

                        k += 1
                        l = d
                    Next
                    ls.Add(s.Trim)
                End If

                ed = pd.EdgesData(mcEdgeTypes.mcetPeaks)
                dC = ed.Distances(0)
                If dC.Length>0 Then
                    s = "Additional #:" & vbTab &
                        String.Format("Position ({0}):", sc) & vbTab &
                        String.Format("Location X ({0}):", "px") & vbTab &
                        String.Format("Location Y ({0}):", "px") & vbTab &
                        String.Format("Intensity ({0}):", ic) & vbTab &
                        String.Format("Distance ({0}):", sc) & vbTab &
                        "Periodic:" & vbNewLine

                    k = 1 : l = 0
                    For Each d In dC
                        index = CInt(d / dStep)
                        dp = pd.Points(index)
                        ar = pd.Values
                        intensity = ar(index)

                        s &= k.ToString & vbTab &
                            Format(d, fs) & vbTab &
                            Format(dp.X, fs) & vbTab &
                            Format(dp.Y, fs) & vbTab &
                            Format(intensity, fs) & vbTab &
                            Format(d - l, fs) & vbTab &
                            GetAnnuliMark(dA, d) & vbNewLine

                        k += 1
                        l = d
                    Next
                    ls.Add(s.Trim)
                End If

            End If

            Return ls.ToArray
        End Get
    End Property

    Private Function GetAnnuliMark(dA() As Double, d As Double) As String
        Dim pd As McProfileData, ed As McEdgesData, ec As McEdgeClass

        If dA.Length > 0 Then
            pd = _lineProfile.ProfileData(0)
            ed = pd.EdgesData(mcEdgeTypes.mcetReference)

            For i As Integer = 0 To dA.Length - 1
                If d < dA(i) Then
                    ec = ed.EdgeClass(0, dA(i))
                    Return ec.Text
                End If
            Next
        End If

        Return ""
    End Function

    Private Function GetFormatString As String
        Dim ahp As MediaCy.Addins.Scripting.AppHostPanel = GetHostPanel(Me.Control)
        Dim dad As Integer = 3

        If ahp IsNot Nothing Then
            dad = ahp.DigitsAfterDecimal
        End If

        Return String.Format("0.{0}", New String("#"c,dad))
    End Function

#End Region

#Region "GetKey, GetHostPanel"

    Private Function GetKey() As String
        Dim ahp As MediaCy.Addins.Scripting.AppHostPanel = GetHostPanel(Me.Control)

        If ahp IsNot Nothing Then
            Return ahp.ControlKey
        End If

        Return Nothing
    End Function

    Private Function GetHostPanel(c As System.Windows.Forms.Control) As MediaCy.Addins.Scripting.AppHostPanel
        If c IsNot Nothing Then
            If TypeOf c Is MediaCy.Addins.Scripting.AppHostPanel Then
                Return c
            Else
                Return GetHostPanel(c.Parent)
            End If
        End If

        Return Nothing
    End Function

#End Region

#Region "Load/Save Settings"

    Private Sub LoadSettings()
        With _app.Settings("Apps", "Otolith")
            _bSpatial = .Get("Spatial", _bSpatial)
            _sSpatial = .Get("SpatialName", _sSpatial)
            _bIntensity = .Get("Intensity", _bIntensity)
            _sIntensity = .Get("IntensityName", _sIntensity)
            _sTransectName = .Get("TransectName", _sTransectName)
            _cTransectColor = .Get("TransectColor", _cTransectColor)
            _dMagnification = .Get("Magnification", _dMagnification)
            _sAnnuliPrefix = .Get("AnnuliPrefix", _sAnnuliPrefix)
            _cAnnuliColor = .Get("AnnuliColor", _cAnnuliColor)
            _sCirculiName = .Get("CirculiName", _sCirculiName)
            _cCirculiColor = .Get("CirculiColor", _cCirculiColor)

        End With
    End Sub

    Private Sub SaveSettings()
        With _app.Settings("Apps", "Otolith")
            .Set("Spatial", _bSpatial)
            .Set("SpatialName", _sSpatial)
            .Set("Intensity", _bIntensity)
            .Set("IntensityName", _sIntensity)
            .Set("TransectName", _sTransectName)
            .Set("TransectColor", _cTransectColor)
            .Set("Magnification", _dMagnification)
            .Set("AnnuliPrefix", _sAnnuliPrefix)
            .Set("AnnuliColor", _cAnnuliColor)
            .Set("CirculiName", _sCirculiName)
            .Set("CirculiColor", _cCirculiColor)

        End With
    End Sub

#End Region

#Region "Init, Cleanup"

    Private Sub Init()
        Dim c As System.Windows.Forms.DataGridViewTextBoxColumn

        dgvAnnuliMarks.Rows.Clear
        dgvAnnuliMarks.Columns.Clear

        c = New System.Windows.Forms.DataGridViewTextBoxColumn
        c.HeaderText = "#:"
        c.Name = "cNumber"
        c.ReadOnly = True
        c.FillWeight = 20
        c.SortMode = System.Windows.Forms.DataGridViewColumnSortMode.NotSortable
        dgvAnnuliMarks.Columns.Add(c)

        c = New System.Windows.Forms.DataGridViewTextBoxColumn
        c.HeaderText = "Name:"
        c.Name = "cName"
        c.ReadOnly = False
        c.FillWeight = 50
        c.SortMode = System.Windows.Forms.DataGridViewColumnSortMode.NotSortable
        dgvAnnuliMarks.Columns.Add(c)

    End Sub

    Private Sub InitLineProfile
        CleanupLineProfile()

        If _app.SelectedData.Image IsNot Nothing Then 'check ActiveDocument
            _lineProfile = MediaCy.Commands.LineProfile.AddIn.LineProfileEngine(_app.ActiveDocument, True)

            _lineProfile.AdjustPictureStrip = True
            _lineProfile.IntensityCalibration = True
            _lineProfile.MonoInterpretation = True
            _lineProfile.SpatialCalibration = True
            _lineProfile.StatisticVisible = True
            _lineProfile.XYVisible = True

            _lineProfile.EdgeClassOnImage = True
            _lineProfile.EdgeClassOnGraph = True
            _lineProfile.EdgeClassOnTable = True

        End If
    End Sub

    Private Sub CleanupLineProfile
        ZoomIn(False)
        SetOverlayTool(OverlayTools.None)

        _lineProfile = Nothing
    End Sub

#End Region

#Region "Steps, UpdateControls"

    Private Sub SetStep(iStep As Integer)
        _iStep = iStep

        UpdateControls()

        gbNavigation.Visible = (3 <= _iStep) And (_iStep <= 4)
        tlpSteps.Visible = (0 <= _iStep) And (_iStep <= 5)
        If tlpSteps.Visible Then pbProgress.Value = _iStep + 1:toolTip1.SetToolTip(pbProgress, String.Format("Step {0} of 6", _iStep + 1))

        gbFile.Visible = (_iStep = 0)
        gbCalibrations.Visible = (_iStep = 1)
        gbProfile.Visible = (_iStep = 2)
        gbAnnuliMarks.Visible = (_iStep = 3)
        gbCirculiMarks.Visible = (_iStep = 4)
        gbExport.Visible = (_iStep = 5)
        gbOptions.Visible = (_iStep = 10)
    End Sub

    Private Sub UpdateControls()
        If Not _bInit Then
            _bInit = True

            Select Case _iStep
                Case 0 : UpdateControls_0()
                Case 1 : UpdateControls_1()
                Case 2 : UpdateControls_2()
                Case 3 : UpdateControls_3() : UpdateNavigation()
                Case 4 : UpdateControls_4() : UpdateNavigation()
                Case 5 : UpdateControls_5()
                Case 10 : UpdateOptions()
            End Select

            _bInit = False
        End If
    End Sub

    Private Sub UpdateControls_0() 'File
        Dim b As Boolean = _lineProfile IsNot Nothing

        ZoomIn(False)
        SetOverlayTool(OverlayTools.None)

        bOpenMeasurements.Enabled = b
        lFileError.Visible = Not b
        bRestart.Enabled = False
        bBack.Enabled = False
        bNext.Enabled = b
    End Sub

    Private Sub UpdateControls_1() 'Calibrations
        Dim b As Boolean = True

        ZoomIn(False)
        SetOverlayTool(OverlayTools.None)

        UpdateSpatial()
        UpdateIntensity()

        If _bSpatial AndAlso String.IsNullOrEmpty(_sSpatial) Then b = False
        If _bIntensity AndAlso String.IsNullOrEmpty(_sIntensity) Then b = False

        lCalibrationsError.Visible = Not b
        bRestart.Enabled = True
        bBack.Enabled = True
        bNext.Enabled = b
    End Sub

    Private Sub UpdateSpatial()
        Dim i As Integer

        cbSpatial.Checked = _bSpatial
        cbSCalib.Enabled = _bSpatial

        cbSCalib.Items.Clear()
        For Each sc As McSpatialCalib In _app.Engine.SpatialCalibs
            i = cbSCalib.Items.Add(sc.Name)
            If sc.Name = _sSpatial Then cbSCalib.SelectedIndex = i
        Next

        If cbSCalib.SelectedIndex = -1 Then
            _sSpatial = ""
            If cbSCalib.Items.Count > 0 Then
                cbSCalib.SelectedIndex = 0
                _sSpatial = cbSCalib.Text
            End If
        End If

        cbSCalib.Items.Add("Edit Calibrations...")
    End Sub

    Private Sub UpdateIntensity()
        Dim i As Integer

        cbIntensity.Checked = _bIntensity
        cbICalib.Enabled = _bIntensity

        cbICalib.Items.Clear()
        For Each ic As McIntensityCalib In _app.Engine.IntensityCalibs
            i = cbICalib.Items.Add(ic.Name)
            If ic.Name = _sIntensity Then cbICalib.SelectedIndex = i
        Next

        If cbICalib.SelectedIndex = -1 Then
            _sIntensity = ""
            If cbICalib.Items.Count > 0 Then
                cbICalib.SelectedIndex = 0
                _sIntensity = cbICalib.Text
            End If
        End If

        cbICalib.Items.Add("Edit Calibrations...")
    End Sub

    Private Sub UpdateControls_2() 'Transect Line
        Dim i As Integer = _lineProfile.FeaturesCount
        Dim b As Boolean = i = 1

        ZoomIn(False)

        If i = 0 Then
            SetOverlayTool(OverlayTools.Polyline)
            lProfile.Text = "Draw the polyline on the otolith."

        ElseIf i = 1 Then
            SetOverlayTool(OverlayTools.Edit)
            lProfile.Text = "Adjust position of the transect line if necessary and click 'Next'."

        Else
            Functions.RemoveAllProfiles()

        End If

        bRemoveProfile.Enabled = i > 0
        lProfileError.Visible = Not b

        bRestart.Enabled = True
        bBack.Enabled = True
        bNext.Enabled = b
    End Sub

    Private Sub UpdateControls_3() 'Annuli Marks
        Dim b As Boolean
        Dim pd As McProfileData, ed As McEdgesData, dis() As Double

        RemoveUnusedClasses()
        UpdateAnnuliMarksCount()
        b = _iAnnuliMarksCount > 0

        dgvAnnuliMarks.Rows.Clear()
        If _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1 Then
            pd = _lineProfile.ProfileData(0)
            ed = pd.EdgesData(mcEdgeTypes.mcetReference)

            If ed.EdgesCount > 0 Then
                dis = ed.Distances(0)
                dgvAnnuliMarks.Rows.Add(dis.Length)
                For i As Integer = 0 To dis.Length - 1
                    dgvAnnuliMarks.Rows(i).Tag = ed.EdgeClass(0, dis(i))
                    dgvAnnuliMarks.Rows(i).Cells(0).Value = (i + 1).tostring
                    dgvAnnuliMarks.Rows(i).Cells(1).Value = ed.EdgeClass(0, dis(i)).Text
                Next
                dgvAnnuliMarks.Currentcell = dgvAnnuliMarks.Rows(dis.Length - 1).Cells(1)
            End If
        End If

        bRemoveAllAnnuliMarks.Text = String.Format("Remove All Marks: {0}", _iAnnuliMarksCount)
        bRemoveAllAnnuliMarks.Enabled = b
        lAnnuliMarksError.Visible = Not b

        SetOverlayTool(OverlayTools.Reference)

        bRestart.Enabled = True
        bBack.Enabled = True
        bNext.Enabled = True
    End Sub

    Private Sub UpdateControls_4() 'Circuli Marks
        Dim b As Boolean

        RemoveUnusedClasses()
        UpdateCirculiMarksCount()
        b = _iCirculiMarksCount > 0

        bRemoveAllCirculiMarks.Text = String.Format("Remove All Marks: {0}", _iCirculiMarksCount)
        bRemoveAllCirculiMarks.Enabled = b
        lCirculiMarksError.Visible = Not b

        SetOverlayTool(OverlayTools.Peaks)

        bRestart.Enabled = True
        bBack.Enabled = True
        bNext.Enabled = True
    End Sub

    Private Sub UpdateControls_5() 'Export
        Dim b As Boolean

        b = _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1
        bSaveMeasurements.Enabled = b
        bExcel.Enabled = b And (_iAnnuliMarksCount + _iCirculiMarksCount) > 0
        bTable.Enabled = b

        ZoomIn(False)
        SetOverlayTool(OverlayTools.None)

        bRestart.Enabled = True
        bBack.Enabled = True
        bNext.Enabled = False
    End Sub

    Private Sub UpdateNavigation()
        Dim b As Boolean

        UpdateZoomPoints()
        b = _ZoomPoints IsNot Nothing AndAlso _ZoomPoints.Count > 1

        gbNavigation.Enabled = b
        cbZoomIn.Checked = _bZoomIn
        udZoom.Value = _dMagnification * 100.0
        bLeft.Enabled = _bZoomIn AndAlso _iZoomStep > 0
        bRight.Enabled = _bZoomIn AndAlso _iZoomStep < _ZoomPoints.Count - 1
        tlpNavigation2.Enabled = _bZoomIn
    End Sub

    Private Sub UpdateOptions()
        tbTransectLineName.Text = _sTransectName
        bTransectLineColor.BackColor = _cTransectColor

        tbFreshWaterPrefix.Text = _sAnnuliPrefix
        bFreshWaterColor.BackColor = _cAnnuliColor

        tbCirculiName.Text = _sCirculiName
        bCirculiColor.BackColor = _cCirculiColor
    End Sub

#End Region

#Region "Events"

#Region "Control, App, Panels"

    Private Sub MyControl_ParentChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyControl.ParentChanged
        Dim k As String = GetKey

        If Not String.IsNullOrEmpty(k) And Control.Parent IsNot Nothing Then
            Functions._AppForm = Me
            Gadgets.OpenLineProfile()

            Init()
            InitLineProfile()
            LoadSettings()
            SetStep(0)

        ElseIf Functions._AppForm IsNot Nothing Then
            Functions._AppForm = Nothing
            Gadgets.CloseLineProfile()
            CleanupLineProfile()
            SaveSettings()

        End If
    End Sub

    Private Sub _panels_ActiveControlChanged(ByVal Control As System.Windows.Forms.Control, ByVal key As String) Handles _panels.ActiveControlChanged
        Dim k As String = GetKey

        If Not String.IsNullOrEmpty(k) AndAlso k = key Then
            UpdateControls()
        End If
    End Sub

    Private Sub _panels_BeforeRemoveControl(ByVal Control As System.Windows.Forms.Control, ByVal key As String, ByRef cancel As Boolean) Handles _panels.BeforeRemoveControl
        Dim k As String = GetKey

        If Not String.IsNullOrEmpty(k) AndAlso key = "LineProfile" Then 'keep LineProfile open
            'MsgBox("Cannot close Line Profile tool while Otolith M is running.", vbOkOnly, "Otolith M")
            cancel = True
        End If
    End Sub

    Private Sub _app_SelectedDataChanged(ByVal Flags As MediaCy.IQL.Application.IMcData.DataTypes) Handles _app.SelectedDataChanged
        Dim k As String = GetKey

        If Not String.IsNullOrEmpty(k) AndAlso Not _bInit Then
            InitLineProfile()
            SetStep(0)
        End If
    End Sub

#End Region

#Region "Line Profile, NotifySink"

    Private Sub _lineProfile_OverlayToolChanged(ByVal overlayTool As MediaCy.Commands.LineProfile.mcOverlayTool) Handles _lineProfile.OverlayToolChanged
        If Not _bInit Then
            SetOverlayTool(_OverlayTool)
        End If
    End Sub

    Private Sub _lineProfile_FeaturesAdded(ByVal mcFeatures As MediaCy.IQL.Features.McFeatures, ByVal iFirstIndex As Integer, ByVal iAddedCount As Integer) Handles _lineProfile.FeaturesAdded
        If Not _bInit Then
            Functions.SetProfileOptions()
            RemoveUnusedClasses()
            UpdateControls()
        End If
    End Sub

    Private Sub _lineProfile_FeaturesRemoved(ByVal mcFeatures As MediaCy.IQL.Features.McFeatures, ByVal Index As Integer) Handles _lineProfile.FeaturesRemoved
        If Not _bInit Then
            RemoveUnusedClasses()
            UpdateControls()
        End If
    End Sub

    Private Sub _lineProfile_FeaturesCoordinatesChanged(ByVal mcFeatures As MediaCy.IQL.Features.McFeatures, ByVal Index As Integer) Handles _lineProfile.FeaturesCoordinatesChanged
        If Not _bInit Then
            Functions.SetProfileOptions()
            RemoveUnusedClasses()
            UpdateControls()
        End If
    End Sub

    Private Sub _lineProfile_EdgesChanged(ByVal iFeature As Integer) Handles _lineProfile.EdgesChanged
        If Not _bInit Then
            RemoveUnusedClasses()
            UpdateControls()
        End If
    End Sub

    Private Sub _lineProfile_EdgesClassAdded(ByVal edgeClass As MediaCy.Commands.LineProfile.McEdgeClass) Handles _lineProfile.EdgesClassAdded
        If Not _bInit Then
            If edgeClass.EdgeType = mcEdgeTypes.mcetReference Then 'Annuli Marks
                _iAnnuliMarksCount += 1
                edgeClass.Text = String.Format("{0} {1}", _sAnnuliPrefix, _iAnnuliMarksCount)
                edgeClass.Color = _cAnnuliColor

            Else 'Circuli Marks
                edgeClass.Text = _sCirculiName
                edgeClass.Color = _cCirculiColor

                _lineProfile.RemoveAllEdgeMeasurements
                _lineProfile.AddEdgeMeasurements(New McEdgeMeasurement(New McEdgeClass(mcEdgeTypes.mcetOrigin), edgeClass))
                _lineProfile.AddEdgeMeasurements(New McEdgeMeasurement(edgeClass, edgeClass))

            End If
        End If
    End Sub

#End Region

#Region "File"

    Private Sub bOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bOpen.Click
        Functions.OpenImageFromFile()
    End Sub

    Private Sub bOpenMeasurements_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles bOpenMeasurements.Click
        Dim s As String = _lineProfile.Document.FileName

        If Not String.IsNullOrEmpty(s) Then
            s = System.IO.Path.GetDirectoryName(s)
        Else
            s = _app.Path(mcPathType.mcptWriteableDocuments)
        End If

        openFileDialog1.FileName = ""
        openFileDialog1.InitialDirectory = s
        openFileDialog1.Multiselect = False
        openFileDialog1.CheckFileExists = True
        openFileDialog1.Filter = _
            IIf(CaliperSupport, _
            "Line Profile State, Caliper Settings|*.lps;*.cps|Line Profile State|*.lps|Caliper Settings|*.cps", _
            "Line Profile State|*.lps")

        If openFileDialog1.ShowDialog = Windows.Forms.DialogResult.OK Then
            _bInit = True
            _lineProfile.Load(openFileDialog1.FileName)
            _bInit = False

            UpdateControls()
        End If
    End Sub

    Private Function CaliperSupport As Boolean
        Dim v As String = _app.Version
        Dim s() As String = v.Split(New String(){"."}, StringSplitOptions.RemoveEmptyEntries)

        If v = "9.0.4729.58" OrElse (CInt(s(0)) = 9 AndAlso (CInt(s(3)) = 1 Or CInt(s(3)) = 2)) Then
            Return False
        End If

        Return True
    End Function

    Private Sub bOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bOptions.Click
        SetStep(10)
    End Sub

#End Region

#Region "Calibrations"

    Private Sub cbSpatial_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbSpatial.CheckedChanged
        If Not _bInit Then
            _bSpatial = cbSpatial.Checked
            UpdateControls()
        End If
    End Sub

    Private Sub cbSCalib_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbSCalib.SelectedIndexChanged
        If Not _bInit Then
            If cbSCalib.SelectedIndex = cbSCalib.Items.Count - 1 Then
                OpenSpatialPanel()

            Else
                _sSpatial = cbSCalib.Text

            End If

            UpdateControls()
        End If
    End Sub

    Private Sub cbIntensity_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbIntensity.CheckedChanged
        If Not _bInit Then
            _bIntensity = cbIntensity.Checked
            UpdateControls()
        End If
    End Sub

    Private Sub cbICalib_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbICalib.SelectedIndexChanged
        If Not _bInit Then
            If cbICalib.SelectedIndex = cbICalib.Items.Count - 1 Then
                OpenInensityPanel()

            Else
                _sIntensity = cbICalib.Text

            End If

            UpdateControls()
        End If
    End Sub

#End Region

#Region "Transect Line"

    Private Sub bLocalZoom_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bLocalZoom.Click
        Gadgets.OpenLocalZoomPanel()
    End Sub

    Private Sub bRemoveProfile_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles bRemoveProfile.Click
        Functions.RemoveAllProfiles()
    End Sub

#End Region

#Region "Annuli Marks"

    Private Sub dgvAnnuliMarks_CellEndEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles dgvAnnuliMarks.CellEndEdit
        Dim ec As McEdgeClass

        If Not _bInit Then
            _bInit = True
            ec = dgvAnnuliMarks.Rows(e.RowIndex).tag
            ec.Text = dgvAnnuliMarks.Rows(e.RowIndex).Cells(e.ColumnIndex).Value.ToString
            _bInit = False
        End If
    End Sub

    Private Sub bRemoveAllAnnuliMarks_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bRemoveAllAnnuliMarks.Click 
        RemoveAnnuliMarks()
    End Sub

#End Region

#Region "Circuli Marks"

    Private Sub bRemoveAllCirculiMarks_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles bRemoveAllCirculiMarks.Click
        RemoveCirculiMarks()
    End Sub

#End Region

#Region "Export"

    Private Sub bSaveMeasurements_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles bSaveMeasurements.Click
        Dim s As String = _lineProfile.Document.FileName, d As String, f As String = Nothing

        If Not String.IsNullOrEmpty(s) Then
            d = System.IO.Path.GetDirectoryName(s)
            f = System.IO.Path.GetFileNameWithoutExtension(s)
        Else
            d = _app.Path(mcPathType.mcptWriteableDocuments)
            f = _lineProfile.Document.DisplayName
        End If

        saveFileDialog1.FileName = f
        saveFileDialog1.InitialDirectory = d
        saveFileDialog1.Filter = "Line Profile State|*.lps"

        If saveFileDialog1.ShowDialog = Windows.Forms.DialogResult.OK Then
            _bInit = True
            _lineProfile.Save(saveFileDialog1.FileName)
            _bInit = False
        End If
    End Sub

    Private Sub bExcel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bExcel.Click
        Functions.ExportExcel()
    End Sub

    Private Sub bTable_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles bTable.Click
        Functions.ExportTable()
    End Sub

    Private Sub bSnapshot_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles bSnapshot.Click
        Functions.CreateSnapshot()
    End Sub

#End Region

#Region "Navigation"

    Private Sub cbZoomIn_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbZoomIn.CheckedChanged
        If Not _bInit Then
            ZoomIn(cbZoomIn.Checked)
            UpdateControls()
        End If
    End Sub

    Private Sub bLeft_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bLeft.Click
        _iZoomStep -= 1
        UpdateControls()
        ZoomIn(True)
    End Sub

    Private Sub bRight_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bRight.Click
        _iZoomStep += 1
        UpdateControls()
        ZoomIn(True)
    End Sub

    Private Sub udZoom_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles udZoom.ValueChanged
        If Not _bInit Then
            _dMagnification = udZoom.Value / 100.0
            UpdateControls()
            ZoomIn(True)
        End If
    End Sub

#End Region

#Region "Options"

    Private Sub bTransectLineColor_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles bTransectLineColor.Click
        colorDialog1.Color = bTransectLineColor.BackColor
        If colorDialog1.ShowDialog(_app.MainForm) = Windows.Forms.DialogResult.OK Then
            bTransectLineColor.BackColor = colorDialog1.Color
        End If
    End Sub

    Private Sub bFreshWaterColor_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bFreshWaterColor.Click
        colorDialog1.Color = bFreshWaterColor.BackColor
        If colorDialog1.ShowDialog(_app.MainForm) = Windows.Forms.DialogResult.OK Then
            bFreshWaterColor.BackColor = colorDialog1.Color
        End If
    End Sub

    Private Sub bCirculiColor_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bCirculiColor.Click 
        colorDialog1.Color = bCirculiColor.BackColor
        If colorDialog1.ShowDialog(_app.MainForm) = Windows.Forms.DialogResult.OK Then
            bCirculiColor.BackColor = colorDialog1.Color
        End If
    End Sub

    Private Sub bOptionsReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bOptionsReset.Click
        tbTransectLineName.Text = ""
        bTransectLineColor.BackColor = Drawing.Color.Red

        tbFreshWaterPrefix.Text = "Year"
        bFreshWaterColor.BackColor = Drawing.Color.LimeGreen

        tbCirculiName.Text = "Growth"
        bCirculiColor.BackColor = Drawing.Color.Yellow
    End Sub

    Private Sub bOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bOk.Click
        Dim s As String, ec As McEdgeClass

        _sTransectName = tbTransectLineName.Text
        _cTransectColor = bTransectLineColor.BackColor
        Functions.SetProfileNameColor()

        s = _sAnnuliPrefix
        _sAnnuliPrefix = tbFreshWaterPrefix.Text
        _cAnnuliColor = bFreshWaterColor.BackColor
        If _lineProfile IsNot Nothing Then 'update Annuli classes
            For Each ec In _lineProfile.EdgeClasses
                If ec.EdgeType = mcEdgeTypes.mcetReference Then
                    If ec.Text.StartsWith(s) Then ec.Text = ec.Text.Replace(s, _sAnnuliPrefix)
                    ec.Color = _cAnnuliColor
                End If
            Next
        End If

        _sCirculiName = tbCirculiName.Text
        _cCirculiColor = bCirculiColor.BackColor
        If _lineProfile IsNot Nothing Then 'update Circuli classes
            For Each ec In _lineProfile.EdgeClasses
                If ec.EdgeType <> mcEdgeTypes.mcetReference Then
                    ec.Text = _sCirculiName
                    ec.Color = _cCirculiColor
                End If
            Next
        End If

        SaveSettings()
        SetStep(0)
    End Sub

    Private Sub bCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bCancel.Click
        SetStep(0)
    End Sub

#End Region

#Region "Restart, Back, Next"

    Private Sub bRestart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bRestart.Click 
        Init()
        SetStep(0)
    End Sub

    Private Sub bBack_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bBack.Click 
        SetStep(_iStep - 1)
    End Sub

    Private Sub bNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bNext.Click 
        If _iStep = 1 Then
            Functions.ApplySpatial()
            Functions.ApplyIntensity()
        End If

        SetStep(_iStep + 1)
    End Sub

#End Region

#End Region

#Region "Functions"

    Private Sub SetOverlayTool(tool As OverlayTools)
        _OverlayTool = tool

        If _lineProfile IsNot Nothing AndAlso _lineProfile.OverlayTool <> _OverlayTool Then
            If (_lineProfile.FeaturesCount = 0 AndAlso _OverlayTool = OverlayTools.Polyline) OrElse _lineProfile.FeaturesCount = 1 Then
                _lineProfile.OverlayTool = CType(_OverlayTool, mcOverlayTool)

            Else
                _OverlayTool = OverlayTools.None
                _lineProfile.OverlayTool = CType(_OverlayTool, mcOverlayTool)

            End If
        End If
    End Sub


    Private Sub UpdateAnnuliMarksCount()
        Dim k As Integer = 0
        Dim pd As McProfileData, ed As McEdgesData

        If _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1 Then
            pd = _lineProfile.ProfileData(0)
            ed = pd.EdgesData(mcEdgeTypes.mcetReference)
            k = ed.EdgesCount
        End If

        _iAnnuliMarksCount = k
    End Sub

    Private Sub RemoveAnnuliMarks()
        Dim pd As McProfileData, ed As McEdgesData

        If _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1 Then
            SetOverlayTool(OverlayTools.None)

            _bInit = True
            pd = _lineProfile.ProfileData(0)
            ed = pd.EdgesData(mcEdgeTypes.mcetReference)
            ed.RemoveEdges()
            _bInit = False

            UpdateControls()
        End If
    End Sub


    Private Sub UpdateCirculiMarksCount()
        Dim k As Integer = 0
        Dim pd As McProfileData, ed As McEdgesData

        If _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1 Then
            pd = _lineProfile.ProfileData(0)
            ed = pd.EdgesData(mcEdgeTypes.mcetPeaks)
            k = ed.EdgesCount
        End If

        _iCirculiMarksCount = k
    End Sub

    Private Sub RemoveCirculiMarks()
        Dim pd As McProfileData, ed As McEdgesData

        If _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1 Then
            SetOverlayTool(OverlayTools.None)

            _bInit = True
            pd = _lineProfile.ProfileData(0)
            ed = pd.EdgesData(mcEdgeTypes.mcetPeaks)
            ed.RemoveEdges()
            _bInit = False

            UpdateControls()
        End If
    End Sub


    Private Sub RemoveUnusedClasses()
        Dim al As New System.Collections.ArrayList
        Dim ec As McEdgeClass

        If _lineProfile IsNot Nothing Then
            For Each ec In _lineProfile.EdgeClasses
                If Not _lineProfile.EdgeClassCollection.ContainsClass(ec) Then
                    al.Add(ec)
                End If
            Next

            For Each ec In al
                _lineProfile.RemoveEdgeClass(ec)
            Next
        End If
    End Sub


    Private Sub UpdateZoomPoints
        Dim pd As McProfileData, l As Integer, dp() As Double, i As Integer
        Dim win As McWindow, w As Integer, h As Integer, wm As Double, hm As Double, dw As Double
        Dim pa As Drawing.PointF, pb As Drawing.PointF, rect As Drawing.RectangleF, p As Drawing.PointF
        Dim l2 As Integer, dp2() As Double

        _ZoomPoints.Clear()
        If _lineProfile IsNot Nothing AndAlso _lineProfile.FeaturesCount = 1 Then
            pd = _lineProfile.ProfileData(0)
            l = pd.Features.GetFeaturePoints(pd.Selector)
            ReDim dp(l * 2 - 1) As Double : pd.Features.GetFeaturePoints(pd.Selector, dp)

            win = _app.ActiveWindow
            w = win.Width : wm = w / _dMagnification
            h = win.Height : hm = h / _dMagnification
            dw = Math.Min(wm, hm) / 2.0

            l2 = CType(pd.Features, Object).mLnLengthUncalib(pd.Selector) / dw : If l2 < 2 Then l2 = 2
            ReDim dp2(l2 * 2 - 1) As Double : pd.Features.GetFeaturePointsEx(pd.Selector, dp2, MediaCy.IQL.Features.mcOutlineType.mcotSetNumbers, l2)

            For i = 0 To l2 - 1
                pa = New Drawing.PointF(dp2(i * 2), dp2(i * 2 + 1))
                _ZoomPoints.Add(New Drawing.PointF(pa.X - wm / 2.0, pa.Y - hm / 2.0)) 'first zoom point is fist profile point
            Next

            i = l - 1 'last point
            pa = New Drawing.PointF(dp(i * 2), dp(i * 2 + 1))
            _ZoomPoints.Add(New Drawing.PointF(pa.X - wm / 2.0, pa.Y - hm / 2.0)) 'last zoom point is last profile point
        End If
    End Sub

    Private Sub ZoomIn(bZoomIn As Boolean)
        If bZoomIn Then
            _bZoomIn = True
            Functions.ViewZoomIn()

        ElseIf _bZoomIn Then
            _bZoomIn = False
            _iZoomStep = 0
            Functions.ViewZoomOut()

        End If
    End Sub

#End Region

End Class
