﻿
Imports MediaCy.IQL.Engine
Imports MediaCy.Viewers.Image
Imports MediaCy.IQL.ObjectManager
Imports MediaCy.IQL.Display.Overlays
Imports MediaCy.Addins.Scripting.Workflow

Public Module Functions

    Public _AppForm As AppForm
    Private _sSpatialUnits As String
    Private _sIntensityUnits As String

    Public Function OpenImageFromFile() As SimpleScript
        OpenImageFromFile = New SimpleScript
        Dim docList1

        With Application.DocumentCommands.Open(OpenImageFromFile)
            .Filenames = Nothing
            .Run(docList1)
        End With

    End Function


    Public Function ApplySpatial() As SimpleScript
        ApplySpatial = New SimpleScript
        Dim var1, spcal1, image1

        With Automate.ScriptingCommands.CodeCommand(ApplySpatial)
            If .Run() Then
                _sSpatialUnits = "px"

                If _AppForm Is Nothing OrElse String.IsNullOrEmpty(_AppForm.SpatialName) Then
                    Exit Function
                Else
                    var1 = _AppForm.SpatialName
                End If
            End If
        End With

        With Measure.Calibration.SpatialCommands.Define(ApplySpatial)
            .Run(var1, spcal1)
        End With

        With Application.DocumentCommands.ActiveImage(ApplySpatial)
            .Run(image1)
        End With

        With Measure.Calibration.SpatialCommands.Apply(ApplySpatial)
            .Run(image1, spcal1)
        End With

        With Automate.ScriptingCommands.CodeCommand(ApplySpatial)
            If .Run() Then
                Dim sc As MediaCy.IQL.Calibrations.McSpatialCalib = spcal1

                If sc IsNot Nothing Then
                    _sSpatialUnits = sc.UnitAbbrev
                End If
            End If
        End With

    End Function

    Public Function ApplyIntensity() As SimpleScript
        ApplyIntensity = New SimpleScript
        Dim var1, incal1, image1

        With Automate.ScriptingCommands.CodeCommand(ApplyIntensity)
            If .Run() Then
                _sIntensityUnits = "lum"

                If _AppForm Is Nothing OrElse String.IsNullOrEmpty(_AppForm.IntensityName) Then
                    Exit Function
                Else
                    var1 = _AppForm.IntensityName
                End If
            End If
        End With

        With Measure.Calibration.IntensityCommands.Define(ApplyIntensity)
            .Run(var1, incal1)
        End With

        With Application.DocumentCommands.ActiveImage(ApplyIntensity)
            .Run(image1)
        End With

        With Measure.Calibration.IntensityCommands.Apply(ApplyIntensity)
            .Run(image1, incal1)
        End With

        With Automate.ScriptingCommands.CodeCommand(ApplyIntensity)
            If .Run() Then
                Dim ic As MediaCy.IQL.Calibrations.McIntensityCalib = incal1

                If ic IsNot Nothing Then
                    _sIntensityUnits = ic.UnitAbbrev
                End If
            End If
        End With

    End Function


    Public Function SetProfileOptions() As SimpleScript
        SetProfileOptions = New SimpleScript
        Dim doc1

        With Application.DocumentCommands.Active(SetProfileOptions)
            .Run(doc1)
        End With

        With Measure.LineProfile.Profile.EdgesCommands.Settings(SetProfileOptions)
            .EdgeType = LineProfile.EdgeTypes.Peaks
            .EdgesVisible = False
            .Index = 0
            .Run(doc1)
        End With

        With Measure.LineProfile.Profile.EdgesCommands.Settings(SetProfileOptions)
            .EdgeType = LineProfile.EdgeTypes.Valleys
            .EdgesVisible = False
            .Index = 0
            .Run(doc1)
        End With

        With Measure.LineProfile.Profile.EdgesCommands.Settings(SetProfileOptions)
            .EdgeType = LineProfile.EdgeTypes.Rising
            .EdgesVisible = False
            .Index = 0
            .Run(doc1)
        End With

        With Measure.LineProfile.Profile.EdgesCommands.Settings(SetProfileOptions)
            .EdgeType = LineProfile.EdgeTypes.Falling
            .EdgesVisible = False
            .Index = 0
            .Run(doc1)
        End With

        With Measure.LineProfile.ProfileCommands.Settings(SetProfileOptions)
            .Index = 0
            .Opacity = 50
            .LabelVisible = False
            .Run(doc1)
        End With

        With Automate.ScriptingCommands.CodeCommand(SetProfileOptions)
            If .Run() Then
                With Measure.LineProfile.ProfileCommands.Settings(SetProfileOptions)
                    .Index = 0
                    If Not String.IsNullOrEmpty(_AppForm.TransectName) Then
                        .Label = _AppForm.TransectName
                        .LabelVisible = True
                    Else
                        .LabelVisible = False
                    End If
                    .Color = _AppForm.TransectColor
                    .Run(doc1)
                End With
            End If
        End With

    End Function

    Public Function SetProfileNameColor() As SimpleScript
        SetProfileNameColor = New SimpleScript
        Dim doc1

        With Application.DocumentCommands.Active(SetProfileNameColor)
            .Run(doc1)
        End With

        With Automate.ScriptingCommands.CodeCommand(SetProfileNameColor)
            If .Run() Then
                With Measure.LineProfile.ProfileCommands.Settings(SetProfileNameColor)
                    .Index = 0
                    If Not String.IsNullOrEmpty(_AppForm.TransectName) Then
                        .Label = _AppForm.TransectName
                        .LabelVisible = True
                    Else
                        .LabelVisible = False
                    End If
                    .Color = _AppForm.TransectColor
                    .Run(doc1)
                End With
            End If
        End With

    End Function

    Public Function RemoveAllProfiles() As SimpleScript
        RemoveAllProfiles = New SimpleScript
        Dim doc1

        With Application.DocumentCommands.Active(RemoveAllProfiles)
            .Run(doc1)
        End With

        With Measure.LineProfile.ProfileCommands.RemoveAll(RemoveAllProfiles)
            .Run(doc1)
        End With

    End Function


    Public Function ViewZoomIn() As SimpleScript
        ViewZoomIn = New SimpleScript
        Dim window1

        With Application.WindowCommands.Active(ViewZoomIn)
            .Run(window1)
        End With

        With Automate.ScriptingCommands.CodeCommand(ViewZoomIn)
            If .Run() Then
                If _AppForm IsNot Nothing Then
                    If window1 IsNot Nothing AndAlso window1.ImageView IsNot Nothing Then
                        With View.ImageViewCommands.Options(ViewZoomIn)
                            .View.Magnification = _AppForm.Magnification
                            .Run(window1)
                        End With

                        Dim w As McWindow
                        Dim p As Integer = window1.ImageView.Pan
                        Dim s As Integer = window1.ImageView.Scroll
                        For Each d As Double In New Double(){0.05, 0.1, 0.2, 0.5, 0.8, 0.9, 0.95, 1.0}
                            window1.ImageView.SetPanScroll(p + (_AppForm.Pan - p) * d, s + (_AppForm.Scroll - s) * d)
                            w = window1: w.Refresh: DoEvents
                        Next
                    End If
                End If
            End If
        End With

    End Function

    Public Function ViewZoomOut() As SimpleScript
        ViewZoomOut = New SimpleScript
        Dim window1

        With Application.WindowCommands.Active(ViewZoomOut)
            .Run(window1)
        End With

        With View.ImageViewCommands.Options(ViewZoomOut)
            .View.AutoZoomMode = MediaCy.IQL.Display.Viewer.mcAutoZoomMode.mazmBestFit
            .Run(window1)
        End With

    End Function


    Public Function ExportExcel() As SimpleScript
        ExportExcel = New SimpleScript

        With Automate.ScriptingCommands.CodeCommand(ExportExcel)
            If .Run() Then
                If _AppForm IsNot Nothing Then
                    Dim s1 As String = _AppForm.Data1, s2() As String = _AppForm.Data2

                    If Not String.IsNullOrEmpty(s1) Then
                        With Measure.Data.ExportCommands.ExportToExcel(ExportExcel)
                            .Options = Export.ExportToExcel.ExportOptions.AppendToBottom
                            .Position = New System.Drawing.Point(1, 1)
                            .Offset = New System.Drawing.Point(0, 0)
                            .Title = True
                            .ColumnHeaders = True
                            .RowHeaders = True
                            .Borders = False
                            .Style = False
                            .Background = False
                            .ExpandedRowsOnly = False
                            .SpecialGroupFormating = False
                            .Direction = Export.ExportToExcel.ExportDirection.Vertical
                            .ColumnHeaders2 = False
                            .RowHeaders2 = True
                            .Run("Otolith M", s1)
                        End With

                        With Measure.Data.ExportCommands.ExportToExcel(ExportExcel)
                            .Options = Export.ExportToExcel.ExportOptions.AppendToBottom
                            .Position = New System.Drawing.Point(1, 1)
                            .Offset = New System.Drawing.Point(0, 0)
                            .Title = False
                            .ColumnHeaders = True
                            .RowHeaders = True
                            .Borders = True
                            .Style = False
                            .Background = False
                            .ExpandedRowsOnly = False
                            .SpecialGroupFormating = False
                            .Direction = Export.ExportToExcel.ExportDirection.Vertical
                            .ColumnHeaders2 = False
                            .RowHeaders2 = True
                            .Run("Otolith M", s2)
                        End With
                    End If
                End If
            End If
        End With

    End Function

    Public Function ExportTable() As SimpleScript
        ExportTable = New SimpleScript

        With Measure.LineProfile.ViewCommands.Table(ExportTable)
            .View = LineProfile.mcProfileView.mcpvTable
            .Run()
        End With

        With Measure.LineProfile.ViewCommands.Excel(ExportTable)
            .Run()
        End With

        With Measure.LineProfile.ViewCommands.Graph(ExportTable)
            .View = LineProfile.mcProfileView.mcpvGraph
            .Run()
        End With

    End Function

    Public Function CreateSnapshot() As SimpleScript
        CreateSnapshot = New SimpleScript
        Dim window1, image1

        With Application.WindowCommands.Active(CreateSnapshot)
            .Run(window1)
        End With

        With Application.WindowCommands.SnapView(CreateSnapshot)
            .Visible = False
            .Run(window1, image1)
        End With

        With Automate.ScriptingCommands.CodeCommand(CreateSnapshot)
            If .Run() Then
                ' User Code Here
                If image1 IsNot Nothing Then
                    .Application.WindowsEx.Add(image1, "", False)
                End If
            End If
        End With

    End Function

End Module
